use 5.20.0;
use strict;
use warnings;

package TestFor::DBIx::Class::Smooth::Schema::ResultSet::Publisher;

# ABSTRACT: ...
# AUTHORITY
our $VERSION = '0.0001';

use TestFor::DBIx::Class::Smooth::Schema::ResultSet -components => [qw/
/];
use DBIx::Class::Smooth::Q;

1;
